@@ -29,6 +29,7 @@ from glue_jupyter.state_traitlets_helpers import GlueState
 from glue_jupyter.bqplot.profile import BqplotProfileView
 from ipyvuetify import VuetifyTemplate
 
+from jdaviz import __version__
 from jdaviz.core.config import read_configuration, get_configuration
 from jdaviz.core.events import (LoadDataMessage, NewViewerMessage, AddDataMessage,
                                 SnackbarMessage, RemoveDataMessage,
@@ -73,6 +74,14 @@ ipyvue.register_component_from_file(None, 'j-plugin-section-header',
                                     os.path.join(os.path.dirname(__file__),
                                                  'components/plugin_section_header.vue'))
 
+ipyvue.register_component_from_file(None, 'j-number-uncertainty',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/number_uncertainty.vue'))
+
+ipyvue.register_component_from_file(None, 'plugin-dataset-select',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_dataset_select.vue'))
+
 ipyvue.register_component_from_file(None, 'plugin-subset-select',
                                     os.path.join(os.path.dirname(__file__),
                                                  'components/plugin_subset_select.vue'))
@@ -81,6 +90,14 @@ ipyvue.register_component_from_file(None, 'plugin-viewer-select',
                                     os.path.join(os.path.dirname(__file__),
                                                  'components/plugin_viewer_select.vue'))
 
+ipyvue.register_component_from_file(None, 'plugin-add-results',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_add_results.vue'))
+
+ipyvue.register_component_from_file(None, 'plugin-auto-label',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_auto_label.vue'))
+
 # Register pure vue component. This allows us to do recursive component instantiation only in the
 # vue component file
 ipyvue.register_component_from_file('g-viewer-tab', "container.vue", __file__)
@@ -146,6 +163,9 @@ class ApplicationState(State):
     tray_items_open = CallbackProperty(
         [], docstring="The plugin(s) opened in sidebar tray area.")
 
+    tray_items_filter = CallbackProperty(
+        '', docstring='User-filter on tray items')
+
     stack_items = ListCallbackProperty(
         docstring="Nested collection of viewers constructed to support the "
                   "Golden Layout viewer area.")
@@ -164,6 +184,7 @@ class Application(VuetifyTemplate, HubListener):
 
     loading = Bool(False).tag(sync=True)
     config = Unicode("").tag(sync=True)
+    vdocs = Unicode("").tag(sync=True)
 
     def __init__(self, configuration=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -829,7 +850,10 @@ class Application(VuetifyTemplate, HubListener):
         data_label = self._build_data_label(data_path, ext=ext)
         data_id = self._data_id_from_label(data_label)
 
-        data_ids = viewer_item['selected_data_items'] if not clear_other_data else []
+        if clear_other_data:
+            self._update_selected_data_items(viewer_item['id'], [])
+
+        data_ids = viewer_item['selected_data_items']
 
         if data_id is not None:
             data_ids.append(data_id)
@@ -1412,6 +1436,7 @@ class Application(VuetifyTemplate, HubListener):
         self._loaded_configuration = config
         # give the vue templates access to the current config/layout
         self.config = config['settings'].get('configuration', 'unknown')
+        self.vdocs = 'latest' if 'dev' in __version__ else 'v'+__version__
 
         self.state.settings.update(config.get('settings'))
 
