@@ -11,21 +11,24 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     app = cubeviz_helper.app
     dc = app.data_collection
     app.add_data(spectrum1d_cube, 'test[FLUX]')
+    app.add_data_to_viewer('flux-viewer', 'test[FLUX]')
 
     mm = MomentMap(app=app)
-    mm._subset_selected = 'None'
-    mm._on_data_updated(None)
-
-    mm._on_data_selected({'new': 'test[FLUX]'})
+    mm.dataset_selected = 'test[FLUX]'
 
     mm.n_moment = 0  # Collapsed sum, will get back 2D spatial image
+    assert mm.results_label == 'moment 0'
     mm.vue_calculate_moment()
 
     assert mm.moment_available
-    assert dc[1].label == 'Moment 0: test[FLUX]'
+    assert dc[1].label == 'moment 0'
 
     assert len(dc.links) == 8
 
+    # label should remain unchanged but raise overwrite warnings
+    assert mm.results_label == 'moment 0'
+    assert mm.results_label_overwrite is True
+
     result = dc[1].get_object(cls=CCDData)
     assert result.shape == (2, 4)  # Cube shape is (2, 2, 4)
 
@@ -42,8 +45,10 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
         mm.vue_save_as_fits()
 
     mm.n_moment = 1
+    assert mm.results_label == 'moment 1'
+    assert mm.results_label_overwrite is False
     mm.vue_calculate_moment()
 
-    assert dc[2].label == 'Moment 1: test[FLUX]'
+    assert dc[2].label == 'moment 1'
 
     assert len(dc.links) == 10
