@@ -4,6 +4,7 @@ import os
 import numpy as np
 from astropy import units as u
 from astropy.io import fits
+from astropy.time import Time
 from astropy.wcs import WCS
 from specutils import Spectrum1D
 
@@ -11,9 +12,9 @@ from jdaviz.core.registries import data_parser_registry
 
 __all__ = ['parse_data']
 
-EXT_TYPES = dict(flux=['flux', 'sci'],
+EXT_TYPES = dict(flux=['flux', 'sci', 'data'],
                  uncert=['ivar', 'err', 'var', 'uncert'],
-                 mask=['mask', 'dq'])
+                 mask=['mask', 'dq', 'quality'])
 
 
 @data_parser_registry("cubeviz-data-parser")
@@ -52,12 +53,20 @@ def parse_data(app, file_obj, data_type=None, data_label=None):
             prihdr = hdulist[0].header
             telescop = prihdr.get('TELESCOP', '').lower()
             filetype = prihdr.get('FILETYPE', '').lower()
+            system = prihdr.get('SYSTEM', '').lower()
             if telescop == 'jwst' and filetype == '3d ifu cube':
                 for ext, viewer_name in (('SCI', 'flux-viewer'),
                                          ('ERR', 'uncert-viewer'),
                                          ('DQ', 'mask-viewer')):
                     data_label = f'{file_name}[{ext}]'
                     _parse_jwst_s3d(app, hdulist, data_label, ext=ext, viewer_name=viewer_name)
+            elif telescop == 'jwst' and filetype == 'r3d' and system == 'esa-pipeline':
+                for ext, viewer_name in (('DATA', 'flux-viewer'),
+                                         ('ERR', 'uncert-viewer'),
+                                         ('QUALITY', 'mask-viewer')):
+                    data_label = f'{file_name}[{ext}]'
+                    _parse_esa_s3d(app, hdulist, data_label, ext=ext, viewer_name=viewer_name)
+
             else:
                 _parse_hdu(app, hdulist, file_name=data_label or file_name)
 
@@ -125,13 +134,13 @@ def _parse_hdu(app, hdulist, file_name=None):
             app.add_data_to_viewer('uncert-viewer', data_label)
 
         if any(x in hdu.name.lower() for x in EXT_TYPES['flux']):
+            # Add flux to top left image viewer
             app.add_data_to_viewer('flux-viewer', data_label)
+            # Add flux to spectrum viewer
             app.add_data_to_viewer('spectrum-viewer', data_label)
 
 
 def _parse_jwst_s3d(app, hdulist, data_label, ext='SCI', viewer_name='flux-viewer'):
-    from specutils import Spectrum1D
-
     # Manually inject MJD-OBS until we can support GWCS, see
     # https://github.com/spacetelescope/jdaviz/issues/690 and
     # https://github.com/glue-viz/glue-astronomy/issues/59
@@ -142,7 +151,6 @@ def _parse_jwst_s3d(app, hdulist, data_label, ext='SCI', viewer_name='flux-viewe
                     hdulist[ext].header['MJD-OBS'] = hdulist[ext].header[key]
                     break
                 else:
-                    from astropy.time import Time
                     t = Time(hdulist[ext].header[key])
                     hdulist[ext].header['MJD-OBS'] = t.mjd
                     break
@@ -165,6 +173,35 @@ def _parse_jwst_s3d(app, hdulist, data_label, ext='SCI', viewer_name='flux-viewe
         app.add_data_to_viewer('spectrum-viewer', data_label)
 
 
+def _parse_esa_s3d(app, hdulist, data_label, ext='DATA', viewer_name='flux-viewer'):
+    if ext == 'QUALITY':  # QUALITY flags have no unit
+        flux = hdulist[ext].data << u.dimensionless_unscaled
+    else:
+        unit = u.Unit(hdulist[ext].header.get('BUNIT', 'count'))
+        flux = hdulist[ext].data << unit
+
+    hdr = hdulist[1].header
+
+    wcs_dict = {
+        'CTYPE1': 'WAVE    ', 'CUNIT1': 'um', 'CDELT1': hdr['CDELT3'] * 1E6,
+        'CRPIX1': hdr['CRPIX3'],
+        'CRVAL1': hdr['CRVAL3'] * 1E6, 'NAXIS1': hdr['NAXIS3'],
+        'CTYPE2': 'DEC--TAN', 'CUNIT2': 'deg', 'CDELT2': hdr['CDELT1'], 'CRPIX2': hdr['CRPIX1'],
+        'CRVAL2': hdr['CRVAL1'], 'NAXIS2': hdr['NAXIS1'],
+        'CTYPE3': 'RA---TAN', 'CUNIT3': 'deg', 'CDELT3': hdr['CDELT2'], 'CRPIX3': hdr['CRPIX2'],
+        'CRVAL3': hdr['CRVAL2'], 'NAXIS3': hdr['NAXIS2']}
+
+    wcs = WCS(wcs_dict)
+    flux = np.moveaxis(flux, 0, -1)
+    flux = np.swapaxes(flux, 0, 1)
+    data = Spectrum1D(flux, wcs=wcs)
+
+    app.add_data(data, data_label)
+    app.add_data_to_viewer(viewer_name, data_label)
+    if viewer_name == 'flux-viewer':
+        app.add_data_to_viewer('spectrum-viewer', data_label)
+
+
 def _parse_spectrum1d_3d(app, file_obj):
     # Load spectrum1d as a cube
 
