@@ -7,21 +7,42 @@ first guesses by the fitting algorithms.
 """
 import numpy as np
 
+import astropy.modeling.models as models
+from astropy import units as u
+
+from jdaviz.models import BlackBody
+
 __all__ = [
-    'initialize'
+    'initialize',
+    'get_model_parameters'
 ]
 
 AMPLITUDE = 'amplitude'
 POSITION = 'position'
 WIDTH = 'width'
 
-model_parameters = {"Gaussian1D": ["amplitude", "stddev", "mean"],
-                    "Const1D": ["amplitude"],
-                    "Linear1D": ["slope", "intercept"],
-                    "PowerLaw1D": ["amplitude", "x_0", "alpha"],
-                    "Lorentz1D": ["amplitude", "x_0", "fwhm"],
-                    "Voigt1D": ["x_0", "amplitude_L", "fwhm_L", "fwhm_G"],
-                    }
+MODELS = {
+     'Const1D': models.Const1D,
+     'Linear1D': models.Linear1D,
+     'Polynomial1D': models.Polynomial1D,
+     'Gaussian1D': models.Gaussian1D,
+     'Voigt1D': models.Voigt1D,
+     'Lorentz1D': models.Lorentz1D,
+     'PowerLaw1D': models.PowerLaw1D,
+     'BlackBody': BlackBody
+     }
+
+
+def get_model_parameters(model_cls, model_kwargs={}):
+    if isinstance(model_cls, str):
+        model_cls = MODELS.get(model_cls)
+
+    if model_cls.__name__ == 'Polynomial1D':
+        # then the parameters are not stored, as they depend on the polynomial order
+        degree = model_kwargs.get('degree', 1)
+        return [f'c{n}' for n in range(degree + 1)]
+
+    return model_cls.param_names
 
 
 def _get_model_name(model):
@@ -82,7 +103,7 @@ class _WideBand1DInitializer(object):
     Parameters
     ----------
     factor: float
-        The scale factor to apply to the amplitutde
+        The scale factor to apply to the amplitude
     """
     def __init__(self, factor=1.0):
         self._factor = factor
@@ -127,7 +148,7 @@ class _LineProfile1DInitializer(object):
     Parameters
     ----------
     factor: float
-        The scale factor to apply to the amplitutde
+        The scale factor to apply to the amplitude
     """
     def __init__(self, factor=1.0):
         self._factor = factor
@@ -203,6 +224,51 @@ class _Sigma_LineProfile1DInitializer(_LineProfile1DInitializer):
         _setattr(instance, name, WIDTH, fwhm / 2.355)
 
 
+class _BlackBodyInitializer:
+    """
+    Initialization that is specific to the BlackBody model.
+
+    Notes
+    -----
+    The fit is sensitive to the scale parameter as if it ever tries
+    to go below zero, the fit will get stuck at 0 (without imposed
+    parameter limits)
+    """
+    def initialize(self, instance, x, y):
+        """
+        Initialize the model
+
+        Parameters
+        ----------
+        instance: `~astropy.modeling.models`
+            The model to initialize.
+
+        x, y: numpy.ndarray
+            The data to use to initialize from.
+
+        Returns
+        -------
+        instance: `~astropy.modeling.models`
+            The initialized model.
+        """
+        y_mean = np.nanmean(y)
+
+        # The y-unit could contain a scale factor (like 1e-7 * FLAM).  We
+        # need to account for this in our dimensionless scale factor guess.
+        # We could make this smarter if we also made a guess for the temperature
+        # based on the peak wavelength/frequency and then estimated the amplitude,
+        # but just getting within an order of magnitude should help significantly
+        y_unit_scaled = y.unit
+        for native_output_unit in instance._native_output_units.values():
+            if y_unit_scaled.is_equivalent(native_output_unit):
+                y_mean = y_mean.to(native_output_unit)
+                break
+
+        instance.scale = y_mean.value * u.dimensionless_unscaled
+
+        return instance
+
+
 def _setattr(instance, mname, pname, value):
     """
     Sets parameter value by mapping parameter name to model type.
@@ -224,10 +290,7 @@ def _setattr(instance, mname, pname, value):
     value: any
         The value to assign.
     """
-    # this has to handle both Quantities and plain floats
     try:
-        setattr(instance, _p_names[mname][pname], value.value)
-    except AttributeError:
         setattr(instance, _p_names[mname][pname], value)
     except KeyError:
         pass
@@ -251,6 +314,7 @@ _initializers = {
     'MexicanHat1D':                _Sigma_LineProfile1DInitializer,  # noqa
     'Trapezoid1D':                 _Width_LineProfile1DInitializer,  # noqa
     'Linear1D':                    _Linear1DInitializer,  # noqa
+    'BlackBody':                   _BlackBodyInitializer,  # noqa
     # 'Spline1D':                   spline.Spline1DInitializer
 }
 
