@@ -2,18 +2,20 @@ import re
 import numpy as np
 
 import astropy.units as u
-from astropy.wcs import WCSSUB_SPECTRAL
-from specutils import Spectrum1D, SpectralRegion
+from specutils import Spectrum1D
 from specutils.utils import QuantityModel
-from traitlets import Any, Bool, List, Unicode, observe
+from traitlets import Bool, List, Unicode, observe
 from glue.core.data import Data
-from glue.core.subset import Subset, RangeSubsetState, OrState, AndState
 from glue.core.link_helpers import LinkSame
-from glue.core.message import SubsetDeleteMessage, SubsetUpdateMessage
 
-from jdaviz.core.events import AddDataMessage, RemoveDataMessage, SnackbarMessage
+from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
-from jdaviz.core.template_mixin import PluginTemplateMixin, SpectralSubsetSelectMixin
+from jdaviz.core.template_mixin import (PluginTemplateMixin,
+                                        SpectralSubsetSelectMixin,
+                                        SubsetSelect,
+                                        DatasetSelectMixin,
+                                        AutoLabel,
+                                        AddResultsMixin)
 from jdaviz.core.custom_traitlets import IntHandleEmpty
 from jdaviz.configs.default.plugins.model_fitting.fitting_backend import fit_model_to_spectrum
 from jdaviz.configs.default.plugins.model_fitting.initializers import (MODELS,
@@ -32,96 +34,69 @@ class _EmptyParam:
 
 
 @tray_registry('g-model-fitting', label="Model Fitting")
-class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
+class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
+                   SpectralSubsetSelectMixin, AddResultsMixin):
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "model_fitting.vue"
-    dc_items = List([]).tag(sync=True)
-    form_valid_data_selection = Bool(False).tag(sync=True)
     form_valid_model_component = Bool(False).tag(sync=True)
 
-    selected_data = Unicode("").tag(sync=True)
+    spatial_subset_items = List().tag(sync=True)
+    spatial_subset_selected = Unicode().tag(sync=True)
 
-    spectral_min = Any().tag(sync=True)
-    spectral_max = Any().tag(sync=True)
-    spectral_unit = Unicode().tag(sync=True)
+    # model components:
+    available_comps = List(list(MODELS.keys())).tag(sync=True)
+    comp_selected = Unicode().tag(sync=True)
+    poly_order = IntHandleEmpty(0).tag(sync=True)
+
+    comp_label = Unicode().tag(sync=True)
+    comp_label_default = Unicode().tag(sync=True)
+    comp_label_auto = Bool(True).tag(sync=True)
+    comp_label_invalid_msg = Unicode().tag(sync=True)
 
-    model_label = Unicode().tag(sync=True)
-    cube_fit = Bool(False).tag(sync=True)
-    temp_name = Unicode().tag(sync=True)
-    temp_model = Unicode().tag(sync=True)
     model_equation = Unicode().tag(sync=True)
+    model_equation_default = Unicode().tag(sync=True)
+    model_equation_auto = Bool(True).tag(sync=True)
+    model_equation_invalid_msg = Unicode().tag(sync=True)
+
     eq_error = Bool(False).tag(sync=True)
     component_models = List([]).tag(sync=True)
     display_order = Bool(False).tag(sync=True)
-    poly_order = IntHandleEmpty(0).tag(sync=True)
 
-    # add/replace results for "fit"
-    add_replace_results = Bool(True).tag(sync=True)
-
-    # selected_viewer for "apply to cube"
-    # NOTE: this is currently cubeviz-specific so will need to be updated
-    # to be config-specific if using within other viewer configurations.
-    viewer_to_id = {'Left': 'cubeviz-0', 'Center': 'cubeviz-1', 'Right': 'cubeviz-2'}
-    viewers = List(['None', 'Left', 'Center', 'Right']).tag(sync=True)
-    selected_viewer = Unicode('None').tag(sync=True)
-
-    available_models = List(list(MODELS.keys())).tag(sync=True)
+    cube_fit = Bool(False).tag(sync=True)
 
     def __init__(self, *args, **kwargs):
         self._spectrum1d = None
         super().__init__(*args, **kwargs)
 
         self._units = {}
-        self.n_models = 0
         self._fitted_model = None
         self._fitted_spectrum = None
         self.component_models = []
         self._initialized_models = {}
         self._display_order = False
-        self.model_label = "Model"
         self._window = None
         self._original_mask = None
         if self.app.state.settings.get("configuration") == "cubeviz":
-            self.cube_fit = True
+            self.spatial_subset = SubsetSelect(self,
+                                               'spatial_subset_items',
+                                               'spatial_subset_selected',
+                                               default_text='Entire Cube',
+                                               allowed_type='spatial')
+        else:
+            self.spatial_subset = None
 
-        self.hub.subscribe(self, AddDataMessage,
-                           handler=self._on_viewer_data_changed)
-        self.hub.subscribe(self, RemoveDataMessage,
-                           handler=self._on_viewer_data_changed)
-        self.hub.subscribe(self, SubsetUpdateMessage,
-                           handler=self._on_viewer_data_changed)
-        self.hub.subscribe(self, SubsetDeleteMessage,
-                           handler=self._on_viewer_data_changed)
+        # when accessing the selected data, access the spectrum-viewer version
+        self.dataset._viewers = ['spectrum-viewer']
+        # require entries to be in spectrum-viewer (not other cubeviz images, etc)
+        self.dataset.add_filter('layer_in_spectrum_viewer')
 
-    def _on_viewer_data_changed(self, msg=None):
-        """
-        Callback method for when data is added or removed from a viewer, or
-        when a subset is created, deleted, or updated. This method receives
-        a glue message containing viewer information in the case of the former
-        set of events, and updates the available data list displayed to the
-        user.
+        self.auto_component_label = AutoLabel(self, 'comp_label', 'comp_label_default',
+                                              'comp_label_auto', 'comp_label_invalid_msg')
+        self.auto_equation = AutoLabel(self, 'model_equation', 'model_equation_default',
+                                       'model_equation_auto', 'model_equation_invalid_msg')
 
-        Notes
-        -----
-        We do not attempt to parse any data at this point, at it can cause
-        visible lag in the application.
-
-        Parameters
-        ----------
-        msg : `glue.core.Message`
-            The glue message passed to this callback method.
-        """
-        self._viewer_id = self.app._viewer_item_by_reference(
-            'spectrum-viewer').get('id')
-
-        viewer = self.app.get_viewer('spectrum-viewer')
-
-        self.dc_items = [layer_state.layer.label
-                         for layer_state in viewer.state.layers
-                         if ((not isinstance(layer_state.layer, Subset)
-                              or not isinstance(layer_state.layer.subset_state,
-                                                (RangeSubsetState, OrState, AndState)))
-                             and layer_state.layer.label not in self.app.fitted_models.keys())]
+        # set the filter on the viewer options
+        self._update_viewer_filters()
 
     def _param_units(self, param, model_type=None):
         """Helper function to handle units that depend on x and y"""
@@ -231,8 +206,8 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         else:
             return False
 
-    @observe("selected_data")
-    def _selected_data_changed(self, event):
+    @observe("dataset_selected", "spatial_subset_selected")
+    def _dataset_selected_changed(self, event=None):
         """
         Callback method for when the user has selected data from the drop down
         in the front-end. It is here that we actually parse and create a new
@@ -246,8 +221,22 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
             IPyWidget callback event object. In this case, represents the data
             label of the data collection object selected by the user.
         """
-        selected_spec = self.app.get_data_from_viewer("spectrum-viewer",
-                                                      data_label=event['new'])
+        if not hasattr(self, 'dataset'):
+            # during initial init, this can trigger before the component is initialized
+            return
+
+        if self.config == 'cubeviz' and self.spatial_subset_selected != 'Entire Cube':
+            # then we're acting on the auto-collapsed data in the spectrum-viewer
+            # of a spatial subset.  In the future, we may want to expose on-the-fly
+            # collapse options... but right now these will follow the settings of the
+            # spectrum-viewer itself
+            selected_spec = self.app.get_data_from_viewer('spectrum-viewer',
+                                                          self.spatial_subset_selected)
+        else:
+            selected_spec = self.dataset.selected_obj
+        if selected_spec is None:
+            return
+
         # Replace NaNs from collapsed Spectrum1D in Cubeviz
         # (won't affect calculations because these locations are masked)
         selected_spec.flux[np.isnan(selected_spec.flux)] = 0.0
@@ -255,11 +244,10 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         # Save original mask so we can reset after applying a subset mask
         self._original_mask = selected_spec.mask
 
-        if self._units == {}:
-            self._units["x"] = str(
-                selected_spec.spectral_axis.unit)
-            self._units["y"] = str(
-                selected_spec.flux.unit)
+        self._units["x"] = str(
+            selected_spec.spectral_axis.unit)
+        self._units["y"] = str(
+            selected_spec.flux.unit)
 
         self._spectrum1d = selected_spec
 
@@ -267,39 +255,49 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         # This is no longer needed for 1D but is preserved for now pending
         # fixes to Cubeviz for multi-subregion subsets
         self._window = None
-        self.spectral_min = selected_spec.spectral_axis[0].value
-        self.spectral_max = selected_spec.spectral_axis[-1].value
-        self.spectral_unit = str(selected_spec.spectral_axis.unit)
 
     @observe("spectral_subset_selected")
     def _on_spectral_subset_selected(self, event):
-        # If "Entire Spectrum" selected, reset based on bounds of selected data
-        if self._spectrum1d is None:
-            # TODO: this should be removed as soon as the data dropdown component is
-            # created and defaults at init
-            return
+        # TODO: does this window not account for gaps?  Should we add the warning?
+        # or can this be removed (see note above in _dataset_selected_changed)
         if self.spectral_subset_selected == "Entire Spectrum":
             self._window = None
-            self.spectral_min = self._spectrum1d.spectral_axis[0].value
-            self.spectral_max = self._spectrum1d.spectral_axis[-1].value
         else:
-            spec_sub = self.spectral_subset.selected_obj
-            unit = u.Unit(self.spectral_unit)
-            if hasattr(spec_sub, "center"):
-                spreg = SpectralRegion.from_center(spec_sub.center.x * unit,
-                                                   spec_sub.width * unit)
-                self._window = (spreg.lower, spreg.upper)
-                self.spectral_min = spreg.lower.value
-                self.spectral_max = spreg.upper.value
+            spectral_min, spectral_max = self.spectral_subset.selected_min_max(self._spectrum1d)
+            self._window = (spectral_min.value, spectral_max.value)
 
-    def vue_model_selected(self, event):
-        # Add the model selected to the list of models
-        self.temp_model = event
-        if event == "Polynomial1D":
+    @observe('comp_selected', 'poly_order')
+    def _update_comp_label_default(self, event={}):
+        abbrevs = {'BlackBody': 'BB', 'PowerLaw': 'PL', 'Lorentz': 'Lo'}
+        abbrev = abbrevs.get(self.comp_selected, self.comp_selected[0].upper())
+        if self.comp_selected == "Polynomial1D":
             self.display_order = True
+            abbrev += f'{self.poly_order}'
         else:
             self.display_order = False
 
+        # append a number suffix to avoid any duplicates
+        ind = 1
+        while abbrev in [cm['id'] for cm in self.component_models]:
+            abbrev = f'{abbrev.split("_")[0]}_{ind}'
+            ind += 1
+
+        self.comp_label_default = abbrev
+
+    @observe('comp_label')
+    def _comp_label_changed(self, event={}):
+        if not len(self.comp_label.strip()):
+            # strip will raise the same error for a label of all spaces
+            self.comp_label_invalid_msg = 'label must be provided'
+            return
+        if self.comp_label in [cm['id'] for cm in self.component_models]:
+            self.comp_label_invalid_msg = 'label already in use'
+            return
+        self.comp_label_invalid_msg = ''
+
+    def _update_model_equation_default(self):
+        self.model_equation_default = '+'.join(cm['id'] for cm in self.component_models)
+
     def _reinitialize_with_fixed(self):
         """
         Reinitialize all component models with current values and the
@@ -325,27 +323,30 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
 
         return temp_models
 
-    def vue_add_model(self, event):
+    def vue_add_model(self, event={}):
         """Add the selected model and input string ID to the list of models"""
+        if not self._spectrum1d:
+            self._dataset_selected_changed()
+
         # validate provided label (only allow "word characters").   These should already be
         # stripped by JS in the UI element, but we'll confirm here (especially if this is ever
         # extended to have better API-support)
-        if re.search(r'\W+', self.temp_name):
-            raise ValueError(f"invalid model component ID {self.temp_name}")
+        if re.search(r'\W+', self.comp_label):
+            raise ValueError(f"invalid model component ID {self.comp_label}")
 
-        if self.temp_name in [cm['id'] for cm in self.component_models]:
-            raise ValueError(f"model component ID {self.temp_name} already in use")
+        if self.comp_label in [cm['id'] for cm in self.component_models]:
+            raise ValueError(f"model component ID {self.comp_label} already in use")
 
-        new_model = {"id": self.temp_name, "model_type": self.temp_model,
+        new_model = {"id": self.comp_label, "model_type": self.comp_selected,
                      "parameters": [], "model_kwargs": {}}
-        model_cls = MODELS[self.temp_model]
+        model_cls = MODELS[self.comp_selected]
 
-        if self.temp_model == "Polynomial1D":
+        if self.comp_selected == "Polynomial1D":
             # self.poly_order is the value in the widget for creating
             # the new model component.  We need to store that with the
             # model itself as the value could change for another component.
             new_model["model_kwargs"] = {"degree": self.poly_order}
-        elif self.temp_model == "BlackBody":
+        elif self.comp_selected == "BlackBody":
             new_model["model_kwargs"] = {"output_units": self._units["y"],
                                          "bounds": {"scale": (0.0, None)}}
 
@@ -369,9 +370,9 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
             initial_values[param_name] = initial_val
 
         initialized_model = initialize(
-            MODELS[self.temp_model](name=self.temp_name,
-                                    **initial_values,
-                                    **new_model.get("model_kwargs", {})),
+            MODELS[self.comp_selected](name=self.comp_label,
+                                       **initial_values,
+                                       **new_model.get("model_kwargs", {})),
             self._spectrum1d.spectral_axis,
             self._spectrum1d.flux)
 
@@ -384,20 +385,56 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
                                             "unit": str(param_quant.unit),
                                             "fixed": False})
 
-        self._initialized_models[self.temp_name] = initialized_model
+        self._initialized_models[self.comp_label] = initialized_model
 
         new_model["Initialized"] = True
         self.component_models = self.component_models + [new_model]
+        # update the default label (likely adding the suffix)
+        self._update_comp_label_default()
+        self._update_model_equation_default()
 
     def vue_remove_model(self, event):
         self.component_models = [x for x in self.component_models
                                  if x["id"] != event]
         del(self._initialized_models[event])
+        self._update_comp_label_default()
+        self._update_model_equation_default()
 
-    def vue_equation_changed(self, event):
+    @observe('model_equation')
+    def _model_equation_changed(self, event):
         # Length is a dummy check to test the infrastructure
+        if len(self.model_equation) == 0:
+            self.model_equation_invalid_msg = 'model equation is required'
+            return
         if len(self.model_equation) > 20:
-            self.eq_error = True
+            self.model_equation_invalid_msg = 'model equation too long'
+            return
+        self.model_equation_invalid_msg = ''
+
+    @observe("dataset_selected", "dataset_items", "cube_fit")
+    def _set_default_results_label(self, event={}):
+        label_comps = []
+        if hasattr(self, 'dataset') and len(self.dataset.labels) > 1:
+            label_comps += [self.dataset_selected]
+        if self.cube_fit:
+            label_comps += ["cube-fit"]
+        label_comps += ["model"]
+        self.results_label_default = " ".join(label_comps)
+
+    @observe("cube_fit")
+    def _update_viewer_filters(self, event={}):
+        if event.get('new', self.cube_fit):
+            # only want image viewers in the options
+            self.add_results.viewer.filters = ['is_image_viewer']
+        else:
+            # only want spectral viewers in the options
+            self.add_results.viewer.filters = ['is_spectrum_viewer']
+
+    def vue_apply(self, event):
+        if self.cube_fit:
+            self.vue_fit_model_to_cube()
+        else:
+            self.vue_model_fitting()
 
     def vue_model_fitting(self, *args, **kwargs):
         """
@@ -432,7 +469,7 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         self._fitted_model = fitted_model
         self._fitted_spectrum = fitted_spectrum
 
-        self.app.fitted_models[self.model_label] = fitted_model
+        self.app.fitted_models[self.results_label] = fitted_model
         self.vue_register_spectrum({"spectrum": fitted_spectrum})
 
         # Update component model parameters with fitted values
@@ -453,8 +490,8 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         if self._warn_if_no_equation():
             return
 
-        if self.selected_data in self.app.data_collection.labels:
-            data = self.app.data_collection[self.selected_data]
+        if self.dataset_selected in self.app.data_collection.labels:
+            data = self.app.data_collection[self.dataset_selected]
         else:  # User selected some subset from spectrum viewer, just use original cube
             data = self.app.data_collection[0]
 
@@ -462,30 +499,18 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         # that the user has selected a pre-existing 1d data object.
         if data.ndim != 3:
             snackbar_message = SnackbarMessage(
-                f"Selected data {self.selected_data} is not cube-like",
+                f"Selected data {self.dataset_selected} is not cube-like",
                 color='error',
                 sender=self)
             self.hub.broadcast(snackbar_message)
             return
 
         # Get the primary data component
-        attribute = data.main_components[0]
-        component = data.get_component(attribute)
-        temp_values = data.get_data(attribute)
+        spec = data.get_object(Spectrum1D, statistic=None)
 
-        # Transpose the axis order
-        values = np.moveaxis(temp_values, 0, -1) * u.Unit(component.units)
-
-        # We manually create a Spectrum1D object from the flux information
-        #  in the cube we select
-        wcs = data.coords.sub([WCSSUB_SPECTRAL])
-        spec = Spectrum1D(flux=values, wcs=wcs)
-
-        # TODO: in vuetify >2.3, timeout should be set to -1 to keep open
-        #  indefinitely
         snackbar_message = SnackbarMessage(
             "Fitting model to cube...",
-            loading=True, timeout=0, sender=self)
+            loading=True, sender=self)
         self.hub.broadcast(snackbar_message)
 
         # Retrieve copy of the models with proper "fixed" dictionaries
@@ -508,30 +533,22 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
         # Save fitted 3D model in a way that the cubeviz
         # helper can access it.
         for m in fitted_model:
-            temp_label = "{} ({}, {})".format(self.model_label, m["x"], m["y"])
+            temp_label = "{} ({}, {})".format(self.results_label, m["x"], m["y"])
             self.app.fitted_models[temp_label] = m["model"]
 
-        # Transpose the axis order back
-        values = np.moveaxis(fitted_spectrum.flux.value, -1, 0)
-
         count = max(map(lambda s: int(next(iter(re.findall(r"\d$", s)), 0)),
                         self.data_collection.labels)) + 1
 
-        label = f"{self.model_label} [Cube] {count}"
+        label = f"{self.results_label} [Cube] {count}"
 
         # Create new glue data object
         output_cube = Data(label=label,
                            coords=data.coords)
-        output_cube['flux'] = values
-        output_cube.get_component('flux').units = \
-            fitted_spectrum.flux.unit.to_string()
+        output_cube['flux'] = fitted_spectrum.flux.value
+        output_cube.get_component('flux').units = fitted_spectrum.flux.unit.to_string()
 
-        # Add to data collection
-        self.app.add_data(output_cube, label)
-        if self.selected_viewer != 'None':
-            # replace the contents in the selected viewer with the results from this plugin
-            self.app.add_data_to_viewer(self.viewer_to_id.get(self.selected_viewer),
-                                        label, clear_other_data=True)
+        self.add_results.add_results_from_plugin(output_cube)
+        self._set_default_results_label()
 
         snackbar_message = SnackbarMessage(
             "Finished cube fitting",
@@ -558,21 +575,12 @@ class ModelFitting(PluginTemplateMixin, SpectralSubsetSelectMixin):
                                                     self.model_equation,
                                                     window=self._window)
 
-        self.n_models += 1
-        label = self.model_label
-        if label in self.data_collection:
-            self.app.remove_data_from_viewer('spectrum-viewer', label)
-            # Remove the actual Glue data object from the data_collection
-            self.data_collection.remove(self.data_collection[label])
-
-        self.app.add_data(spectrum, label)
+        self.add_results.add_results_from_plugin(spectrum)
+        self._set_default_results_label()
 
         # Link the result spectrum to the reference data of the spectrum viewer
 
         ref_data = self.app.get_viewer('spectrum-viewer').state.reference_data
-        data_id = ref_data.world_component_ids[0]
-        model_id = self.app.session.data_collection[label].world_component_ids[0]
+        data_id = ref_data.world_component_ids[-1]
+        model_id = self.app.session.data_collection[self.results_label].world_component_ids[0]
         self.app.session.data_collection.add_link(LinkSame(data_id, model_id))
-
-        if self.add_replace_results:
-            self.app.add_data_to_viewer('spectrum-viewer', label)
