@@ -57,6 +57,9 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         self.display_uncertainties = False
         self.display_mask = False
 
+        # Change collapse function to sum
+        self.state.function = 'sum'
+
     def _on_subset_create(self, msg):
         for layer in self.state.layers:
             if layer.layer.label == msg.subset.label:
@@ -476,5 +479,5 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         # Make it so y axis label is not covering tick numbers.
         self.figure.axes[1].label_offset = "-50"
 
-        # # Set Y-axis to scientific notation
+        # Set Y-axis to scientific notation
         self.figure.axes[1].tick_format = '0.1e'
