@@ -2,10 +2,11 @@ import numpy as np
 
 from jdaviz.core.helpers import ConfigHelper
 from jdaviz.core.events import SnackbarMessage
+from jdaviz.configs.default.plugins.line_lists.line_list_mixin import LineListMixin
 
 
-class Specviz2d(ConfigHelper):
-    """SpecViz2D Helper class"""
+class Specviz2d(ConfigHelper, LineListMixin):
+    """Specviz2D Helper class"""
 
     _default_configuration = "specviz2d"
 
@@ -106,13 +107,11 @@ class Specviz2d(ConfigHelper):
 
         spectrum_1d_label : str
             String representing the label for the data item loaded via
-            ``onedspectra``. Can be a list of strings representing data labels
-            for each item in ``data_obj`` if  ``data_obj`` is a list.
+            ``spectrum_1d``.
 
         spectrum_2d_label : str
             String representing the label for the data item loaded via
-            ``twodspectra``. Can be a list of strings representing data labels
-            for each item in ``data_obj`` if  ``data_obj`` is a list.
+            ``spectrum_2d``.
 
         """
         if spectrum_2d_label is None:
